#ifndef CLUSTER_ENUM
#define CLUSTER_ENUM

#include <gecode/driver.hh>
#include <gecode/int.hh>
#include <gecode/float.hh>
#include <gecode/set.hh>
#include <gecode/minimodel.hh>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <time.h>
#include <sys/resource.h>
#include <vector>
#include <math.h>
#include <algorithm> 
#include "clusterOptions.h"

using namespace Gecode;
using namespace std;

class ClusterEnum : public Space {
protected:
  const ClusterOptions& options;
  IntVarArray G; // G[i]: 1..k,  G[i]: class of i
  FloatVar objVar;  

public:

 ClusterEnum(const ClusterOptions& opt, int n, 
	 vector< vector<double> >& dd, vector<double>& bestVal, 
	 vector<int>& orderSP, vector<int>& rorderSP, vector<int>& sizePart,
	 vector< vector<int> >& clgraph, vector< vector<int> >& cc,
	 vector<int>& rorder, vector<int>& ccindex,
	 vector< vector<double> >& attrMat) : 
  //Space(opt),
    options(opt),
    G(*this, n, 0, opt.k()-1), 
    objVar(*this, 0, Float::Limits::max)  
  {
    postConstraints(dd,bestVal,
		    orderSP, rorderSP, sizePart, clgraph,
		    cc, rorder, ccindex, attrMat);
    postBranch(dd, bestVal);
  }

  ClusterEnum(bool share, ClusterEnum& s) : 
    Space(share, s), options(s.options) {
    G.update(*this, share, s.G);
    objVar.update(*this, share, s.objVar);
  }

  virtual Space* copy(bool share) {
    return new ClusterEnum(share,*this);
  }


  /* virtual void constrain(const Space& sol) { */
  /*   if (options.obj() > 0) { */
  /*     const ClusterEnum& _sol=static_cast<const Cluster&>(sol);       */
  /*     rel(*this, objVar < _sol.objVar.min()); */
  /*   } */
  /* } */

  virtual void print() const;
  void saveSolution(vector<int>& finalClass, vector<double>& bestVal);
  
  void postConstraints(vector< vector<double> >& dd,
		       vector<double>& bestVal,
		       vector<int>& orderSP, vector<int>& rorderSP, 
		       vector<int>& sizePart,
		       vector< vector<int> >& clgraph, vector< vector<int> >& cc,
		       vector<int>& rorder, vector<int>& ccindex,
		       vector< vector<double> >& attrMat);
  void postBranch( vector< vector<double> >& dd, vector<double>& bestVal);
  double objVal();
};


#endif
